# coding: utf-8
from __future__ import absolute_import, unicode_literals

from concentrator.smev3.applicationrequest.steps import (CheckOrderStep,
                                                         ExistingDeclarationStep,
                                                         MultipleDelclarationsStep,
                                                         NewDeclarationStep,
                                                         RequestValidationStep,
                                                         Response, SNILSStep)
from concentrator.smev3.base.tests.utils import copy_from
from kinder.core.declaration.tests.factory_declaration import DeclarationF

from .base import ApplicationTC
from .factories import RequestDeclarationF


class StepsTC(ApplicationTC):

    def assertNextStep(self, step, next_step_class):
        self.assertTrue(isinstance(step, next_step_class))

    def test_step_1(self):
        """Шаг 1. Поиск совпадения по идентификатору заявления orderId."""

        self.assertNextStep(CheckOrderStep(self.application).next(), NewDeclarationStep)

        decl = DeclarationF.create(client_id=self.application.orderId)
        self.assertNextStep(CheckOrderStep(self.application, decl).next(), ExistingDeclarationStep)

    def test_step_2(self):
        """Шаг 2. Сравнение данных заявления в ЭДС и данных в запросе."""

        next_step = ExistingDeclarationStep(
            self.application,
            DeclarationF.create(
                children__firstname=self.application.ChildInfo.ChildName,
                children__surname=self.application.ChildInfo.ChildSurname)
        ).next()
        self.assertNextStep(next_step, Response)

        next_step = ExistingDeclarationStep(
            self.application,
            DeclarationF.create(children__firstname='A')
        ).next()
        self.assertNextStep(next_step, Response)

    def test_step_3(self):
        """Шаг 3. Подача нового заявления, поиск в существующих заявках."""

        declaration = DeclarationF.create()
        next_step = NewDeclarationStep(
            RequestDeclarationF(), declaration
        ).next()
        self.assertNextStep(next_step, RequestValidationStep)

        next_step = NewDeclarationStep(
            RequestDeclarationF._copy_from(declaration),
            declaration,
        ).next()
        self.assertNextStep(next_step, ExistingDeclarationStep)

        copy_from(declaration, fields=(
            'children.firstname',
            'children.surname',
            'children.patronymic',
            'children.date_of_birth',
        ))
        next_step = NewDeclarationStep(
            RequestDeclarationF._copy_from(declaration),
            declaration,
        ).next()
        self.assertNextStep(next_step, MultipleDelclarationsStep)

    def test_step_4(self):
        """Шаг 4. Поиск по параметру 'Свидетельство о рождении'."""

        declaration = DeclarationF.create()
        next_step = MultipleDelclarationsStep(
            RequestDeclarationF(),
            DeclarationF._meta.model.objects.all()
        ).next()
        self.assertNextStep(next_step, SNILSStep)

        copy_from(declaration, fields=(
            'children.firstname',
            'children.surname',
            'children.patronymic',
            'children.date_of_birth',
            'children.dul_number',
            'children.dul_series',
            'children.dul_date',
        ))
        next_step = MultipleDelclarationsStep(
            RequestDeclarationF._copy_from(declaration),
            DeclarationF._meta.model.objects.all()
        ).next()
        self.assertNextStep(next_step, Response)

    def test_step_5(self):
        """Шаг 5. Поиск заявок по СНИЛС."""

        declaration = DeclarationF.create()
        next_step = SNILSStep(
            RequestDeclarationF(),
            DeclarationF._meta.model.objects.all()
        ).next()
        self.assertNextStep(next_step, RequestValidationStep)

        next_step = SNILSStep(
            RequestDeclarationF._copy_from(declaration),
            DeclarationF._meta.model.objects.all()
        ).next()
        self.assertNextStep(next_step, ExistingDeclarationStep)
